import six

class ResultProxy(object):
    def __init__(self, job):
        self.job = job

    @property
    def status(self):
        return self.job.status()

    @property
    def job_id(self):
        return self.job.job_id

    @property
    def size(self):
        return self.job._result_size

    @property
    def description(self):
        return self.job._hive_result_schema

    def __iter__(self):
        return self.job.result()

    def to_csv(self, path_or_file):
        def _to_csv(f):
            # TODO: need optimization
            f.write(",".join([c[0] for c in self.description]))
            f.write("\n")
            for row in self:
                f.write(",".join([str(c) if c else '' for c in row]) + "\n")

        if isinstance(path_or_file, six.string_types):
            with open(path_or_file, 'w') as f:
                return _to_csv(f)
        else:
            return _to_csv(path_or_file)

    def to_dataframe(self):
        # TODO: need optimization
        import pandas as pd
        return pd.DataFrame(iter(self), columns=[c[0] for c in self.description])



from .test_helper import TestConfig
from .task import DatabaseTask
from .task import TableTask
from .task import Query
from .targets.result import ResultTarget

from unittest import TestCase
from nose.tools import eq_, raises

import luigi

test_config = TestConfig(
    jobs = [
        {
            'job_id': 1,
            'status': 'success',
            'size': 20,
            'description': [['cnt', 'int']],
            'rows': [[5000]],
        }
    ]
)

# DatabaseTask

class TestDatabaseTask(DatabaseTask):
    config = test_config

class DatabaseTaskTestCase(TestCase):
    def test_create(self):
        task = TestDatabaseTask('test_db')
        task.run()

# TableTask

class TestTableTask(TableTask):
    config = test_config

class TableTaskTestCase(TestCase):
    def test_create(self):
        task = TestTableTask('test_db', 'test_table')
        task.run()

# Query

class TestQuery(Query):
    config = test_config
    type = 'hive'
    database = 'sample_datasets'
    def query(self):
        return 'select count(1) cnt from www_access'

class QueryTestCase(TestCase):
    def setUp(self):
        test_config.setUp()

    def tearDown(self):
        test_config.tearDown()

    def test_simple(self):
        class SimpleTestQuery(TestQuery):
            pass
        task = SimpleTestQuery()
        task.run()

    def test_with_output(self):
        class OutputTestQuery(TestQuery):
            def output(self):
                return ResultTarget(test_config.get_tmp_path('{0}.job'.format(self)))
        task = OutputTestQuery()
        task.run()

    def test_with_dependency(self):
        class DependencyTestQuery(TestQuery):
            def output(self):
                return ResultTarget(test_config.get_tmp_path('{0}.job'.format(self)))
        class DependencyTestResult(luigi.Task):
            def requires(self):
                return DependencyTestQuery()
            def output(self):
                return LocalTarget(test_config.get_tmp_path('{0}.csv'.format(self)))
        task = DependencyTestResult()
        task.run()



import luigi
from luigi_td.config import get_config
from luigi_td.client import ResultProxy

import json
import os

import logging
logger = logging.getLogger('luigi-interface')

class ResultTarget(luigi.Target):
    def __init__(self, path, result_url=None, config=None):
        self.path = path
        self.result_url = result_url
        self.config = config or get_config()

    # Job result handling

    def save_result_state(self, result):
        state_dir = os.path.dirname(self.path)
        if state_dir != '' and not os.path.exists(state_dir):
            os.makedirs(state_dir)
        with open(self.path, 'w') as f:
            state = {'job_id': result.job_id, 'status': result.status}
            json.dump(state, f)

    def load_result_state(self):
        with open(self.path) as f:
            return json.load(f)

    @property
    def job_id(self):
        return self.load_result_state()['job_id']

    @property
    def status(self):
        return self.load_result_state()['status']

    @property
    def result(self):
        if not hasattr(self, '_result'):
            client = self.config.get_client()
            self._result = ResultProxy(client.job(self.job_id))
        return self._result

    # Luigi support

    def exists(self):
        if not os.path.exists(self.path):
            return False
        if self.status in ['success']:
            return True
        if self.status in ['error', 'killed']:
            return False
        # TODO: should we wait for the end of the job?
        client = self.config.get_client()
        if client.job(self.job_id).success():
            return True
        return False

    # For subclasses

    def get_result_url(self):
        return self.result_url



from ..test_helper import TestConfig
from .s3 import S3ResultTarget

from unittest import TestCase
from nose.tools import eq_, raises

from six.moves.urllib.parse import urlparse
from six.moves.urllib.parse import parse_qs
from six.moves.urllib.parse import unquote as url_unquote

test_config = TestConfig()

class TableauServerResultTargetTestCase(TestCase):
    def setUp(self):
        test_config.setUp()

    def tearDown(self):
        test_config.tearDown()

    def test_default(self):
        target = S3ResultTarget(test_config.get_tmp_path('result.job'))
        target.aws_access_key_id = 'AWS_ACCESS_KEY_ID'
        target.aws_secret_access_key = 'AWS_SECRET_ACCESS_KEY'
        target.bucket = 'test-bucket'
        target.path = 'test-prefix/test.tsv'
        url = urlparse(target.get_result_url())
        params = parse_qs(url.query)
        eq_(url.scheme, 's3')
        eq_(url.path, '/{0}/{1}'.format(target.bucket, target.path))
        eq_(url_unquote(url.username), target.aws_access_key_id)
        eq_(url_unquote(url.password), target.aws_secret_access_key)
        eq_(params.get('format'), ['tsv'])
